package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.device.EnumDeviceType;
import com.xy.dto.DeviceEventMsgDto;
import com.xy.dto.DeviceInfoDto;
import com.xy.entity.DeviceEventMsg;
import com.xy.entity.SysCodeConfigureRedis;
import com.xy.enums.SysCodeConfigureEnum;
import com.xy.mapper.DeviceEventMsgMapper;
import com.xy.service.factory.device.service.DeviceFactory;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 机器-消息表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-09
 */
@Service
@Api(tags = "机器-消息表")
@AllArgsConstructor(onConstructor_ = @Lazy)
public class DeviceEventMsgServiceImpl extends ServiceImpl<DeviceEventMsgMapper, DeviceEventMsg> implements DeviceEventMsgService {

    private DeviceInfoServiceImpl deviceInfoService;

    @Override
    @ApiOperation("添加")
    public R save(DeviceEventMsgDto.Save save) {
        DeviceInfoDto.Vo deviceInfo = deviceInfoService.obj(new DeviceInfoDto.Obj().setDeviceId(save.getDeviceId())).getData();
        if (deviceInfo == null) {
            return R.fail("设备不存在");
        }
        return FactoryUtils.getService(() -> deviceInfo.getDeviceType() == EnumDeviceType.N_5.getIntCode() ? 2 : 1, DeviceFactory.class)
                .eventMsg(save);
    }

    @Override
    @ApiOperation("分页查询")
    public R<PageBean<DeviceEventMsgDto.Vo>> page(@RequestBody DeviceEventMsgDto.Page page) {
        List<String> codes = getCodes(page.getCode());
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceEventMsg> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceEventMsg.class)
                .ge(DeviceEventMsg::getCreateTime, page.getBeginCreateTime())
                .le(DeviceEventMsg::getCreateTime, page.getEndCreateTime())
                .in(DeviceEventMsg::getCode, codes)
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceEventMsg::getId);
        IPage<DeviceEventMsg> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        PageBean<DeviceEventMsgDto.Vo> voPageBean = toPageBean(DeviceEventMsgDto.Vo.class, iPage);
        //反显设备名称
        if (Emptys.check(voPageBean.getRecords())) {
            List<Long> deviceIdList = voPageBean.getRecords().stream().map(DeviceEventMsgDto.Vo::getDeviceId).collect(Collectors.toList());
            Map<Long, String> deviceMap = deviceInfoService.getDeviceNameList(new DeviceInfoDto.DeviceIdDto().setDeviceId(deviceIdList)).getData();
            Map<String, SysCodeConfigureRedis> stringSysCodeConfigureRedisMap = SysCodeConfigureUtils.get(SysCodeConfigureEnum.D01.getCode());
            voPageBean.getRecords().forEach(i -> {
                i.setDeviceName(deviceMap.get(i.getDeviceId()));
                SysCodeConfigureRedis sysCodeConfigureRedis = stringSysCodeConfigureRedisMap.get(i.getCode());
                if (sysCodeConfigureRedis != null) {
                    i.setSysCodeConfigureRedis(sysCodeConfigureRedis);
                }
            });
        }
        return R.ok(voPageBean);
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceEventMsgDto.Vo>> list(@RequestBody DeviceEventMsgDto.SelectList selectList) {
        List<String> codes = getCodes(selectList.getCode());
        LambdaQueryWrapper<DeviceEventMsg> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceEventMsg.class)
                .in(DeviceEventMsg::getId, selectList.getIds())
                .in(DeviceEventMsg::getDeviceId, selectList.getDeviceIds())
                .in(DeviceEventMsg::getCode, codes)
                .build();
        List<DeviceEventMsg> list = list(lambdaQueryWrapper);
        Map<String, SysCodeConfigureRedis> stringSysCodeConfigureRedisMap = SysCodeConfigureUtils.get(SysCodeConfigureEnum.D01.getCode());
        List<DeviceEventMsgDto.Vo> vos = copy(DeviceEventMsgDto.Vo.class, list);
        vos.forEach(vo -> {
            SysCodeConfigureRedis sysCodeConfigureRedis = stringSysCodeConfigureRedisMap.get(vo.getCode());
            if (sysCodeConfigureRedis != null) {
                vo.setSysCodeConfigureRedis(sysCodeConfigureRedis);
            }
        });
        return R.ok(vos);
    }

    private List<String> getCodes(String code) {
        List<String> codes = new ArrayList<>();
        if (Emptys.check(code)) {
            Map<String, SysCodeConfigureRedis> stringSysCodeConfigureRedisMap = SysCodeConfigureUtils.get(code);
            if (Emptys.check(stringSysCodeConfigureRedisMap)) {
                stringSysCodeConfigureRedisMap.forEach((code2, sysCodeConfigureRedis) -> codes.add(code2));
            } else {
                codes.add(code);
            }
        }
        return codes;
    }
}